﻿using System;
using RLNET;

namespace RogueLearning
{
    class Program
    {
        private static RLRootConsole _rootConsole;
        private const int screenWidth = 80;
        private const int screenHeight = 24;
        private static int playerX;
        private static int playerY;
        private static GameCounters _counters;

        static void Main(string[] args)
        {
            _rootConsole = new RLRootConsole("ascii_8x8.png", screenWidth, screenHeight, 8, 8, 2f, "Rogue Learning");

            _rootConsole.OnLoad += RootConsole_OnLoad;
            _rootConsole.Render += RootConsole_Render;
            _rootConsole.Update += RootConsole_Update;

            _counters = new GameCounters();
            _rootConsole.Run();
        }

        private static void RootConsole_Update(object sender, UpdateEventArgs e)
        {
            bool userAction = false;

            // Handle keyboard input
            RLKeyPress key = _rootConsole.Keyboard.GetKeyPress();
            if (key != null)
            {
                switch (key.Key)
                {
                    case RLKey.Up:
                        userAction = true;
                        playerY -= 1;
                        break;

                    case RLKey.Left:
                        userAction = true;
                        playerX -= 1;
                        break;

                    case RLKey.Down:
                        userAction = true;
                        playerY += 1;
                        break;

                    case RLKey.Right:
                        userAction = true;
                        playerX += 1;
                        break;

                    case RLKey.Q:
                    case RLKey.Escape:
                        _rootConsole.Close();
                        break;

                    case RLKey.D:
                        _counters.Visible = !_counters.Visible;
                        break;
                }
            }

            // Turn based events only if userAction is true
            if (userAction)
            {
                ProcessTurn();
            }

            // Update frame timers and counters
            ProcessFrame();
        }

        private static void RootConsole_Render(object sender, UpdateEventArgs e)
        {
            _rootConsole.Clear();

            _rootConsole.SetChar(playerX, playerY, '@');
            if (_counters.Visible)
            {
                _rootConsole.Print(1, 1, $"Current tick: {_counters.Tick} Current turn: {_counters.Turn}", RLColor.White);
                _rootConsole.Print(1, 2, $"Time since start: {_counters.SecondsSinceStart:.000} Current FPS: {_counters.CurrentFPS:.0 fps} Average FPS: {_counters.AverageFPS:.0} fps", RLColor.White);
            }

            _rootConsole.Draw();
        }

        private static void RootConsole_OnLoad(object sender, EventArgs e)
        {
            playerX = screenWidth / 2;
            playerY = screenHeight / 2;
        }

        private static void ProcessFrame()
        {
            _counters.NewFrame();
        }

        private static void ProcessTurn()
        {
            // Ensure player stays on the screen
            if (playerX < 0)
                playerX = 0;
            if (playerX > screenWidth - 1)
                playerX = screenWidth - 1;
            if (playerY < 0)
                playerY = 0;
            if (playerY > screenHeight - 1)
                playerY = screenHeight - 1;
            _counters.NewTurn();
        }

    }
}